using System;
using System.Collections.Generic;
using System.Web.Mvc;
using System.Linq;
using ActivateYourGlutes.Data;
using ActivateYourGlutes.Infrastructure;
using ActivateYourGlutes.Infrastructure.Logging;
using ActivateYourGlutes.Payloads.MeasurementTypePayloads;
using ActivateYourGlutes.Services;
using ActivateYourGlutes.Services.Model;

namespace ActivateYourGlutes.Controllers
{
    public partial class MeasurementTypeController : LocalisingController
    {
        private readonly IMeasurementTypeProvider _measurementTypeProvider;

        public MeasurementTypeController(
            IMeasurementTypeProvider measurementTypeProvider,
            IAccountProvider accountProvider,
            ILogger logger) : base(accountProvider, logger)
        {
            _measurementTypeProvider = measurementTypeProvider;
        }

        [Authorize(Roles = "User")]
        public virtual ActionResult Index()
        {
            ViewData.Model = _measurementTypeProvider.MeasurementTypes(CurrentAccount);
            return View();
        }

        [Authorize(Roles = "User")]
        public virtual ActionResult Create()
        {
            var payload = new EditCreatePayload()
                                 {
                                     MeasurementType = new MeasurementType(),
                                     ValueTypes = ValueTypes()
                                 };
            payload.ValueTypes.First().Selected = true;
            ViewData.Model = payload;
            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        [Authorize(Roles="User")]
        public virtual ActionResult Create(MeasurementType measurementType)
        {
            if (!ModelState.IsValid)
            {
                ViewData.Model = new EditCreatePayload()
                {
                    MeasurementType = measurementType,
                    ValueTypes = ValueTypes()
                };
                return View();
            }
            try
            {
                _measurementTypeProvider.AddToAccount(CurrentAccount, measurementType);
                return RedirectToAction(MVC.MeasurementType.Index());
            }
            catch (ValidationException ex)
            {
                ViewData.Model = new EditCreatePayload()
                {
                    MeasurementType = measurementType,
                    ValueTypes = ValueTypes()
                };
                ex.AddToModelState(ViewData.ModelState);
                return View();
            }
        }

        private IEnumerable<SelectListItem> ValueTypes()
        {
            return
                _measurementTypeProvider.ValueTypes().Select(
                    vt => new SelectListItem() {Selected = false, Text = vt.Description, Value = vt.Code});
        }

        [Authorize(Roles="User")]
        public virtual ActionResult ConfirmDelete(int id)
        {
            try
            {
                ViewData.Model = _measurementTypeProvider.GetById(CurrentAccount, id);
            }
            catch (AuthorizationException)
            {
                return RedirectToAction(MVC.Account.Unauthorized());
            }
            return View();
        }

        [Authorize(Roles = "User")]
        public virtual ActionResult Delete(int id, string response)
        {
            if (response.Equals("Yes"))
            {
                try
                {
                    _measurementTypeProvider.Delete(CurrentAccount, id);
                }
                catch (AuthorizationException)
                {
                    return RedirectToAction(MVC.Account.Unauthorized());
                }
            }
            return RedirectToAction(MVC.MeasurementType.Index());
        }
    }
}
